/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive;

/**
 * An Action impacts the environnement of a community.
 * This is the reason while it is always executed during the inactive part
 *   of an instant.
 * There is no deterministic reverse to a given Action.
 */
public interface Action {

	/**
	 * Values for status.
	 */
	public static final int STATUS_PROPOSED = 0;
	public static final int STATUS_EXECUTABLE = 1;
	public static final int STATUS_EXECUTED = 2;
	
	/**
	 * Returns the current status of this Action.
	 * It is one of the STATUS_* constants.
	 * @return the current status of this Action.
	 */
	public int getStatus();
	
	/**
	 * Returns true if this Action is the same than the given one,
	 *   false elsewhere.
	 * @param action an Action.
	 * @return true if this Action is the same than the given one,
	 *   false elsewhere.
	 */
	public boolean equals(Action action);
	
	/**
	 * Executes this Action.
	 * @param key a key that allows this Action.
	 * @exception Exception is throwed if the Action fails.
	 */	
	public void execute(long key) throws Exception;
}
