/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive;

/**
 * An Event belongs to a ReactiveCommunity.
 * An Event occurs or doesn't occur during a particular instant of its ReactiveCommunity.
 * An Event can have a ReactiveEntity owner, which can (re)name or unvalidate it.
 */
public interface Event {

	/**
	 * Returns <tt>true</tt> if this Event is currently valid, false elsewhere.
	 * @return <tt>true</tt> if this Event is currently valid, false elsewhere.
	 */
	public boolean isValid();
	
	/**
	 * @return the ReactiveCommunity to which this Event belongs to,
	 */
	public ReactiveCommunity getCommunity();
	
	/**
	 * Returns the name of this Event if any, null elsewhere.
	 * An Event name is unique within its ReactiveCommunity.
	 * Event's names are internalized.
	 * A no more valid Event has no name.
	 */
	public String getName();
	
	/**
	 * A base Event is atomic and instantaneous : it is obtained at the same instant
	 * 	of its generation.
	 * @return <tt>true</tt> if this Event is a base Event, false elsewhere.
	 */
	public boolean isBaseEvent();
	
	/**
	 * A Message is an Event that may have attachments and is available at the beginning
	 * 	of the instant that follows immediately the instant of its generation.
	 * @return <tt>true</tt> if this Event is a Message, false elsewhere.
	 */
	public boolean isMessage();
}
