/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.agent;

import org.xmloperator.reactive.exception.BadCharInNameException;
import org.xmloperator.reactive.exception.ReactiveException;

import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.Message;
import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.TermsAttachment;

import org.xmloperator.reactive.facet.ExitFacet;

/**
 * This Agent provides regular clock events during idle periods of time.
 */
public class ClockAgent extends AgentImpl {
	
	protected static final String CLOCK = "clock".intern();

	static {
		BadCharInNameException.verifyName(CLOCK);
	}
	
	protected final ExitFacet exitFacet;
	protected int timing_ms = 1000;

	public ClockAgent() {
		this.exitFacet = new ExitFacet();
		this.exitFacet.attach(this);
	}
	
	public ClockAgent(long globalExitKey, long thisAgentExitKey,
			int timing_ms) {
		this();
		this.setExitKeys(globalExitKey, thisAgentExitKey);
		this.setTiming(timing_ms);
	}
	
	public void setExitKeys(long globalExitKey, long thisAgentExitKey) {
		this.exitFacet.setExitKeys(globalExitKey, thisAgentExitKey);
	}
	
	public void setTiming(int timing_ms) {
		if (timing_ms <= 0)
			throw new IllegalArgumentException();
		this.timing_ms = timing_ms;
	}
	
	///////////////////////
	// Agent implementation
	///////////////////////
	
	public void init(LaunchingContext launchingContext) {
		super.init(launchingContext);
		this.exitFacet.init(launchingContext);
	}

	public boolean continues() {
		return !this.exitFacet.isExit();
	}

	public int idlePeriodDuration_ms() {
		return this.timing_ms;
	}
	
	public void endOfIdlePeriod() {
		if (this.reac.getCommunity() == null)
			throw new ReactiveException(
					"Needs a destination community");
		Event e_clock = this.reac.searchEvent(CLOCK);
		this.reac.generateEvent(e_clock);
	}
	
	public boolean processMessage(Message message, Object attachment) {
		return (attachment instanceof TermsAttachment)
			&& this.exitFacet.processServiceMessage(message.getName(),
					(TermsAttachment)attachment);
	}
}
