/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.agent;

import org.xmloperator.reactive.exception.BadCharInNameException;
import org.xmloperator.reactive.exception.PrivilegiedAccessException;
import org.xmloperator.reactive.exception.ReactiveException;

//import org.xmloperator.reactive.Message;

import org.xmloperator.reactive.util.TermsAttachmentFactory;

/**
 * The mission of this Agent consists in inviting all the agents
 *   of the ReactiveCommunity to which it is connected to exit.
 */
public class GlobalExitAgent extends AgentImpl {

	protected static final String COMMAND = "command".intern();
	protected static final String EXIT = "exit".intern();
	
	static {
		BadCharInNameException.verifyName(COMMAND);
		BadCharInNameException.verifyName(EXIT);
	}

	protected long globalExitKey = 0;

	public GlobalExitAgent() {
	}
	
	public GlobalExitAgent(long globalExitKey) {
		this.globalExitKey = globalExitKey;
	}
	
	public void setExitKey(long globalExitKey) {
		if (this.globalExitKey != 0)
			throw new PrivilegiedAccessException();
		this.globalExitKey = globalExitKey;
	}
	
	///////////////////////
	// Agent implementation
	///////////////////////
	
	public final void terminate() {
		super.terminate();
		if (this.reac.getCommunity() == null)
			throw new ReactiveException(
					"Needs a destination community");
		this.reac.generateMessage(
				this.reac.searchMessage(true, COMMAND),
				TermsAttachmentFactory.newTermsAttachment(
						EXIT, new Long(this.globalExitKey)));
	}
}
