/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

/**
 * Rebuilder of a TranslatableObject from a SAX data flow.
 */
public interface Rebuilder {
	
	/**
	 * SAX ContentHandler : notify the start of an element.
	 * @param namespaceURI
	 * @param localName
	 * @param rawName
	 * @param atts
	 * @throws SAXException
	 */
	public void startElement(
			String namespaceURI, String localName, String rawName,
			Attributes atts) throws SAXException;
	
	/**
	 * SAX ContentHandler : notify the end of an element.
	 * @param namespaceURI
	 * @param localName
	 * @param rawName
	 * @return the rebuided TranslatableObject if this end is the last one,
	 *   null elsewhere.
	 * @throws SAXException
	 */
	public TranslatableObject endElement(
			String namespaceURI, String localName, String rawName)
			throws SAXException;
	
	/**
	 * SAX ContentHandler : notify text.
	 * @param ch
	 * @param start
	 * @param length
	 * @throws SAXException
	 */
	public void characters (char ch[], int start, int length)
			throws SAXException;
}
