/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.action;

import org.xmloperator.reactive.Action;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.ParentElement;
import org.xmloperator.reactive.element.RWElement;
import org.xmloperator.reactive.element.ElementWritingAccess;

import org.xmloperator.reactive.element.action.ElementActionImpl;

/**
 * Action of inserting an Element.
 */
public final class ElementInsertAction extends ElementActionImpl {
	
	public final Element insertedElement;
	
	public ElementInsertAction(ElementWritingAccess elementWritingAccess,
			Element referenceElement, Element insertedElement,
			int status) {
		super(elementWritingAccess, referenceElement, status);
		this.insertedElement = insertedElement;
	}
	
	public boolean isElementInsertAction() {
		return true;
	}

	///////////////////////////////
	// ElementAction implementation
	///////////////////////////////
	
	public ParentElement getTouchedParentElement() {
		return this.referenceElement.getParent();
	}
	
	////////////////////////
	// Action implementation
	////////////////////////
	
	public boolean equals(Action action) {
		if (action instanceof ElementInsertAction) {
			ElementInsertAction otherAction = (ElementInsertAction)action;
			return this.referenceElement == otherAction.referenceElement
				&& this.insertedElement.equals(otherAction.insertedElement);
		}
		return false;
	}

	public void execute(long key) {
		RWElement rwElement =
			(RWElement)this.elementWritingAccess.rwElement(
				this.referenceElement, key);
		rwElement.insertBefore(this.insertedElement);
	}
	
	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(super.toString());
		sb.append("insertBefore");
		sb.append(this.insertedElement);
		return sb.toString();
	}
}
