/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.constraint;

import org.xmloperator.reactive.element.exception.ChildrenConstraintException;

import org.xmloperator.reactive.element.ChildrenConstraint;
import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.ParentElement;

/**
 * This ChildrenConstraint fixes the min and max count of children.
 */
public final class ChildCountChildrenConstraint
		implements ChildrenConstraint {
	
	public final int minChildCount;
	public final int maxChildCount;
	
	/**
	 * Constructor.
	 * @param minChildCount a minimum count of child.
	 * @param maxChildCount a maximum count of child or -1 for no limit.
	 */
	public ChildCountChildrenConstraint(
			int minChildCount, int maxChildCount) {
		if (minChildCount < 0)
			throw new IllegalArgumentException();
		this.minChildCount = minChildCount;
		this.maxChildCount = maxChildCount;
	}
	
	/////////////////////
	// ChildrenConstraint
	/////////////////////
	
	public void verifyConstraint(
			ParentElement parentElement,
			Element addedOrRemovedChildElement, boolean isChildAdded)
			throws ChildrenConstraintException {
		int childCount = parentElement.getChildCount();
		if (isChildAdded) {
			if (this.maxChildCount >= 0
					&& childCount >= this.maxChildCount) {
//				System.err.println("Add one element to " + childCount);
				throw new ChildrenConstraintException(this);
			}
		}
		else {
			if (childCount <= this.minChildCount) {
//				System.err.println("Remove one element to " + childCount);
				throw new ChildrenConstraintException(this);
			}
		}
	}
	
	////////////////////////
	// Object implementation
	////////////////////////

	public String toString() {
		return "C(" + this.minChildCount + ","
			+ (this.maxChildCount >= 0
				? String.valueOf(this.maxChildCount)
				: "")
			+ ")";
	}
}
