/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.impl;

import java.util.List;
import java.util.ArrayList;

import org.xmloperator.reactive.exception.PrivilegiedAccessException;

import org.xmloperator.reactive.Board;
import org.xmloperator.reactive.ReactiveCommunity;

import org.xmloperator.reactive.element.RootElement;
import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.ElementWritingAccess;
import org.xmloperator.reactive.element.Namespace;
import org.xmloperator.reactive.element.RWElement;

/**
 * Board implementation based on a ParentElement.
 */
public final class ElementBoardImpl extends ParentElementImpl
		implements Board, RootElement, ElementWritingAccess {
	
	static final String BOARD_LOCAL_NAME = "board";
	
	final ReactiveCommunity reactiveCommunity;
	private final long writingKey;
	private final List namespaceList = new ArrayList();
	
	public ElementBoardImpl(ReactiveCommunity reactiveCommunity,
			long writingKey) {
		super(Element.REACTIVE_NAMESPACE, BOARD_LOCAL_NAME);
		this.reactiveCommunity = reactiveCommunity;
		this.writingKey = writingKey;
		this.namespaceList.add(
				new Namespace(Element.REACTIVE_NAMESPACE, "reac"));
	}
	
	public ElementBoardImpl(ReactiveCommunity reactiveCommunity,
			Element[] children, long writingKey) {
		this(reactiveCommunity, writingKey);
		if (children != null)
			this.addChildren(children);
	}
	
	///////////////////////
	// Board implementation
	///////////////////////
	
	public ReactiveCommunity getCommunity() {
		return this.reactiveCommunity;
	}
	
	/////////////////////////////
	// RootElement implementation
	/////////////////////////////
	
	public int namespaceCount() {
		return this.namespaceList.size();
	}
	
	public Namespace getNamespace(int index) {
		return (Namespace)this.namespaceList.get(index);
	}
	
	public void addNamespace(Namespace namespace) {
		this.namespaceList.add(namespace);
	}

	//////////////////////////////////////
	// ElementWritingAccess implementation
	//////////////////////////////////////
	
	public RWElement rwElement(Element element, long writingKey) {
		if (writingKey != this.writingKey)
			throw new PrivilegiedAccessException();
		return ((ElementImpl)element).getRWElement();
	}
}
