/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.spec;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.Rebuilder;
import org.xmloperator.reactive.element.TranslatableObject;

/**
 * Rebuilder of a ChildElementSpecification.
 */
public final class ChildElementSpecRebuilder implements Rebuilder {

	private Rebuilder parentElementSpecRebuilder = null;
	private ElementSpecification parentElementSpecification = null;
	private String namespaceURI = null;
	private String localName = null;
	private boolean isNameElementEnded = false;

	public ChildElementSpecRebuilder() {
	}
	
	///////////////////////////
	// Rebuilder implementation
	///////////////////////////
	
	public void startElement(
			String namespaceURI,
			String localName,
			String rawName,
			Attributes atts)
			throws SAXException {
		if (this.parentElementSpecRebuilder != null)
			this.parentElementSpecRebuilder.startElement(
				namespaceURI, localName, rawName, atts);
		else if (this.parentElementSpecification != null){
			if (this.localName != null)
				throw new SAXException(new IllegalStateException());
			this.namespaceURI = namespaceURI;
			this.localName = localName;
		}
		else
			try {
				if (namespaceURI != Element.REACTIVE_NAMESPACE)
					throw new IllegalStateException();
				this.parentElementSpecRebuilder =
					ElementSpecRebuilderFactory.newElementSpecRebuilder(
						localName);
			}
			catch (Exception exception) {
				throw new SAXException(exception);
			}
	}

	public TranslatableObject endElement(
			String namespaceURI,
			String localName,
			String rawName)
			throws SAXException {
		if (this.localName != null && !this.isNameElementEnded) {
			this.isNameElementEnded = true;
			return null;
		}
		if (this.parentElementSpecification != null) {
			if (namespaceURI != Element.REACTIVE_NAMESPACE
					|| (!localName.equals("child")
						&& !(localName.equals("childName")
							&& this.localName != null)))
				throw new SAXException(new IllegalStateException());
			return new ChildElementSpecification(
				this.parentElementSpecification,
				this.namespaceURI,
				this.localName);
		}
		if (this.parentElementSpecRebuilder == null)
			throw new SAXException(new IllegalStateException());
		this.parentElementSpecification =
			(ElementSpecification)this.parentElementSpecRebuilder.endElement(
				namespaceURI, localName, rawName);
		if (this.parentElementSpecification != null)
			this.parentElementSpecRebuilder = null;
		return null;
	}

	public void characters(char[] ch, int start, int length)
			throws SAXException {
		if (this.parentElementSpecRebuilder != null)
			this.parentElementSpecRebuilder.characters(ch, start, length);
		else
			throw new SAXException(new IllegalStateException());
	}
}
