/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.spec;

import java.util.Map;
import java.util.HashMap;

import org.xmloperator.reactive.element.exception.UndefinedElementSpecificationException;

import org.xmloperator.reactive.element.Rebuilder;

/**
 * Factory of Rebuilder of ElementSpecification.
 */
public abstract class ElementSpecRebuilderFactory {
	
	private static final Map fromNameToClass = new HashMap();
	
	static {
		fromNameToClass.put("root", RootElementSpecRebuilder.class);
		fromNameToClass.put("name", ElementNameSpecRebuilder.class);
		fromNameToClass.put("child", ChildElementSpecRebuilder.class);
		fromNameToClass.put("childName", ChildElementSpecRebuilder.class);
		fromNameToClass.put("includingDescendance",
			IncludingDescendanceElementSpecRebuilder.class);
		fromNameToClass.put("and", AndElementSpecRebuilder.class);
		fromNameToClass.put("andNot", AndElementSpecRebuilder.class);
	}
	
	public static final Rebuilder newElementSpecRebuilder(String name)
			throws UndefinedElementSpecificationException {
		Class rebuilderClass = (Class)fromNameToClass.get(name);
		if (rebuilderClass != null)
			try {
				return (Rebuilder)rebuilderClass.newInstance();
			}
			catch (Exception exception) {}
		throw new UndefinedElementSpecificationException();
	}
}
