/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.facet.event;

import org.xmloperator.reactive.Event;
/**
 * Boolean condition based on Event occuring or not occuring.
 * It is either
 *  + the true value,
 *  + the false value,
 *  + a positive base Event,
 *  + a negative base Event,
 *  + an AND between two BoolConditions,
 *  + an OR between two BoolConditions
 */
public interface BoolCondition {
	
	/**
	 * Returns true in the case of a true or a false value.
	 * @return true in the case of a true or a false value.
	 */
	public boolean isResolved();
	
	/**
	 * Returns true in the case of a true value, false else where.
	 * @return true in the case of a true value, false else where.
	 */
	public boolean booleanValue();
	
	/**
	 * Returns the Event in the case of a (positive or negative) base Event,
	 *   null elsewhere.
	 * @return the Event in the case of a (positive or negative) base Event,
	 *   null elsewhere.
	 */
	public Event getBaseEvent();
	
	/**
	 * Returns true in the case of a negative base Event, null elsewhere.
	 * @return true in the case of a negative base Event, null elsewhere.
	 */
	public boolean isNegativeEvent();
	
	/**
	 * Returns the first condition in the case of an AND or an OR condition,
	 *   null elsewhere.
	 * @return the first condition in the case of an AND or an OR condition,
	 *   null elsewhere.
	 */
	public BoolCondition getCondition1();
	
	/**
	 * Returns the second condition in the case of an AND or an OR condition,
	 *   null elsewhere.
	 * @return the second condition in the case of an AND or an OR condition,
	 *   null elsewhere.
	 */
	public BoolCondition getCondition2();
	
	/**
	 * Returns true in the case of an AND condition, false elsewhere.
	 * @return true in the case of an AND condition, false elsewhere.
	 */
	public boolean isAndNode();
	
	/**
	 * Returns true if this BoolCondition contains at least
	 *   a NegativeEventBoolCondition, false elsewhere.
	 * @return true if this BoolCondition contains at least
	 *   a NegativeEventBoolCondition, false elsewhere.
	 */
	public boolean containsNegativeEvent();
	
	/**
	 * Returns what becomes this BoolCondition after the occurence of an Event.
	 * @param event an occuring Event.
	 * @return what becomes this BoolCondition after the occurence of an Event.
	 */
	public BoolCondition restAfterEvent(Event event);
	
	/**
	 * Returns the resolution of this BoolCondition at the end-of-instant.
	 * @return the resolution of this BoolCondition at the end-of-instant.
	 */
	public BoolCondition restAtEndOfInstant();
}
