/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.facet.event;

import org.xmloperator.reactive.exception.ReactiveException;

import org.xmloperator.reactive.Agent;
import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.ReactiveMachine;

import org.xmloperator.reactive.facet.FacetImpl;
import org.xmloperator.reactive.facet.event.BoolCondition;

/**
 * This facet is responsible of generating an event (the deducted one)
 *   at each instant a given boolean expression
 *   of event occuring or not occuring is true.
 */
public final class ConditionalEventFacet extends FacetImpl {
	
	private BoolCondition condition = null;
	private Event deductedEvent = null;
	private boolean isDiffered = false;
	private BoolCondition rest = null;
	
	public ConditionalEventFacet() {
	}
	
	public ConditionalEventFacet(BoolCondition condition, Event deductedEvent,
			boolean isDiffered) {
		this.setExpression(condition, deductedEvent, isDiffered);
	}
	
	public void setExpression(BoolCondition condition, Event deductedEvent,
			boolean isDiffered) {
		if (condition == null || deductedEvent == null
				|| !deductedEvent.isBaseEvent())
			throw new IllegalArgumentException();
		if (!isDiffered && condition.containsNegativeEvent())
			throw new ReactiveException(
				"The deducted event cannot be generated at the same instant.");
		this.condition = condition;
		this.deductedEvent = deductedEvent;
		this.isDiffered = isDiffered;
		if (this.trace != null)
			this.trace.println(this.reac.getEntityId() + " will generate "
				+ this.deductedEvent + (this.isDiffered ? " differed" : "")
				+ " on " + this.condition);
	}
	
	public void clearExpression() {
		this.condition = null;
		this.deductedEvent = null;
		this.rest = null;
	}
	
	///////////////////////
	// Facet implementation
	///////////////////////

	public void init(LaunchingContext launchingContext) {
		super.init(launchingContext);
		{
			String agentProfile = (String)launchingContext.getArgumentValue(
					Agent.ARG_AGENT_PROFILE);
			if (agentProfile.indexOf(ReactiveMachine.PROFILE_EVENT_GENERATION)
					< 0)
				throw new ReactiveException(
			"This agent has not the profile for generating events");
		}
	}
	
	public void startOfInstant() {
		if (this.condition != null) {
			if (this.isDiffered
					&& this.rest != null
					&& this.rest.restAtEndOfInstant().booleanValue())
				this.reac.generateEvent(this.deductedEvent);
			this.rest = this.condition;
		}
	}
	
	public boolean processEvent(Event event) {
		if (this.rest != null && !this.rest.isResolved()) {
			this.rest = this.rest.restAfterEvent(event);
			if (!this.isDiffered
					&& this.rest.isResolved()
					&& this.rest.booleanValue())
				this.reac.generateEvent(this.deductedEvent);
			return true;
		}
		return false;
	}
}
