/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.facet.event;

import org.xmloperator.reactive.Event;

/**
 * BoolCondition corresponding to an OR condition.
 */
public final class OrBoolCondition extends BoolCondition2Impl {
	
	public OrBoolCondition(BoolCondition condition1, BoolCondition condition2) {
		super(condition1, condition2);
	}

	//////////////////////////
	// BoolNode implementation
	//////////////////////////
	
	public BoolCondition restAfterEvent(Event event) {
		BoolCondition condition1 = this.condition1.restAfterEvent(event);
		BoolCondition condition2 = this.condition2.restAfterEvent(event);
		if (condition1.isResolved())
			return condition1.booleanValue() ? condition1 : condition2;
		if (condition2.isResolved())
			return condition2.booleanValue() ? condition2 : condition1;
		return condition1 == this.condition1 && condition2 == this.condition2
			? this
			: new OrBoolCondition(condition1, condition2);
	}

	public BoolCondition restAtEndOfInstant() {
		BoolCondition condition1 = this.condition1.restAtEndOfInstant();
		BoolCondition condition2 = this.condition2.restAtEndOfInstant();
		if (!condition1.isResolved() || !condition2.isResolved())
			throw new IllegalStateException();
		return condition1.booleanValue() || condition2.booleanValue()
			? (BoolCondition)TrueBoolCondition.TRUE
			: FalseBoolCondition.FALSE;
	}
	
	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		return "(" + this.condition1 + " or " + this.condition2 + ")";
	}
}
