/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.impl;

import java.util.List;
import java.util.ArrayList;

import org.xmloperator.reactive.Message;
import org.xmloperator.reactive.ReactiveEntity;
import org.xmloperator.reactive.ReactiveCommunity;

/**
 * Message implementation.
 */
final class MessageImpl extends EventImpl implements Message {
	
	private final boolean isSenderCertified;
	private final boolean isActionMessage;
	private final List attachmentList = new ArrayList();
	private final Object[][] attachmentss = new Object[10][];

	MessageImpl(ReactiveCommunity community, ReactiveEntity owner,
			boolean isSenderCertified, boolean isAction) {
		super(community, owner);
		if (isAction && !isSenderCertified)
			throw new IllegalArgumentException();
		this.isSenderCertified = isSenderCertified;
		this.isActionMessage = isAction;
		for (int k = 0; k < this.attachmentss.length; k++)
			this.attachmentss[k] = new Object[k];
	}

	Object[] getAttachments() {
		int attachmentCount = this.attachmentList.size();
		Object[] attachments = (attachmentCount < this.attachmentss.length)
			? this.attachmentss[attachmentCount]
			: new Object[attachmentCount];
		this.attachmentList.toArray(attachments);
		return attachments;
	}

	void clearAttachments() {
		this.attachmentList.clear();
	}

	void addAttachment(final Object attachment) {
		synchronized (this) {
			this.attachmentList.add(attachment);
		}
	}

	void unvalidate() {
		super.unvalidate();
		this.attachmentList.clear();
	}

	////////////////////////
	// Object implementation
	////////////////////////

	public String toString() {
		return (this.isSenderCertified
				? (this.isActionMessage ? "MCA" : "MC")
				: "M")
			+ this.id;
	}
	
	///////////////////////
	// Event implementation
	///////////////////////
	
	public final boolean isBaseEvent() {
		return false;
	}
	
	public final boolean isMessage() {
		return true;
	}

	/////////////////////////
	// Message implementation
	/////////////////////////
	
	public final boolean isSenderCertified() {
		return this.isSenderCertified;
	}

	public final boolean isActionMessage() {
		return this.isActionMessage;
	}
}
