/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.test;

import java.io.PrintStream;

import org.xmloperator.reactive.Agent;
import org.xmloperator.reactive.CrashingReport;
import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.MachineKeys;
//import org.xmloperator.reactive.Message;
import org.xmloperator.reactive.ReactiveCommunity;
import org.xmloperator.reactive.ReactiveMachine;

import org.xmloperator.reactive.impl.LaunchingContextImpl;
import org.xmloperator.reactive.impl.ReactiveFactoryImpl;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.ElementWritingAccess;
import org.xmloperator.reactive.element.ParentElement;

import org.xmloperator.reactive.element.impl.ParentElementImpl;

import org.xmloperator.reactive.element.action.ElementActionImpl;
import org.xmloperator.reactive.element.action.ElementAppendAction;

import org.xmloperator.reactive.util.End;

import org.xmloperator.reactive.facet.ExitFacet;

import org.xmloperator.reactive.facet.event.BoolCondition;
import org.xmloperator.reactive.facet.event.PositiveEventBoolCondition;
import org.xmloperator.reactive.facet.event.NegativeEventBoolCondition;
import org.xmloperator.reactive.facet.event.OrBoolCondition;
import org.xmloperator.reactive.facet.event.ConditionalEventFacet;

import org.xmloperator.reactive.agent.AgentImpl;
import org.xmloperator.reactive.agent.ClockAgent;
import org.xmloperator.reactive.agent.FacetsAgent;
import org.xmloperator.reactive.agent.GlobalExitAgent;

/**
 * Test of the ReactiveMachine.
 */
public abstract class Test {
	
	/**
	 * A good value for the system breathe duration.
	 */
	private static final int BREATHE_DURATION_MS = 20;
	
	/**
	 * ---- Trace specification
	 * C/c : reactive Community (ReactiveCommunityImpl)
	 * R/r : Reactive entity (ReactiveEntityImpl)
	 * T/t : agent Thread (AgentThread)
	 * M/m : reactive Machine (ReactiveMachineImpl)
	 * A   : Agents
	 */
	private static final String DEFAULT_TRACE_SPEC = "crtMA";
	private static final boolean AUTO_FLUSH = false;
	
	public static void main(String[] args) {
		PrintStream trace = new PrintStream(System.out, AUTO_FLUSH);
		CrashingReport crashingReport = new CrashingReportImpl();
		long t1 = System.currentTimeMillis();
		allTests(trace, crashingReport);
		long t2 = System.currentTimeMillis();
		if (crashingReport.isAllOk()) {
			trace.println("No agent crashes");
			trace.println("Uses " + (t2 - t1) + " ms");
		}
		else
			System.err.println("Some agent has crashed");
		System.exit(0);
	}
	
	public static void allTests(PrintStream trace, CrashingReport report) {
		ReactiveFactoryImpl.setTrace(trace, DEFAULT_TRACE_SPEC);
		test_3_stops_and_2_suspends(trace, report);
		test_globalExit(trace, report);
		test_clock(trace, report);
		test_conditionalEvent(trace, report);
		ReactiveFactoryImpl.setTrace(trace, "crtm");
		test_Syracuse(trace, report);
		ReactiveFactoryImpl.setTrace(trace, DEFAULT_TRACE_SPEC);
		test_board(trace, report);
	}
	
	public static void test_3_stops_and_2_suspends(PrintStream trace,
			CrashingReport crashingReport) {
		if (trace != null)
			trace.println("--- test_3_stops_and_2_suspends");
		MachineKeys machineKeys = new MachineKeys(); 
		ReactiveMachine reactiveMachine =
			ReactiveFactoryImpl.REACTIVE_FACTORY.newReactiveMachine(
					BREATHE_DURATION_MS, crashingReport, machineKeys);
		Agent agent = new AgentImpl() {
			protected ReactiveCommunity newCommunity() {
				return this.reac.newCommunity("test");
			}
			public boolean continues() {
				this.reac.stop();
				this.reac.stop();
				this.reac.stop();
				this.reac.suspend();
				this.reac.suspend();
				this.reac.closeCommunity();
				return false;
			}
		};
		reactiveMachine.launchAgent(
				machineKeys.agentLaunchingKey,
				agent,
				null,
				ReactiveMachine.PROFILE_FULL);
		reactiveMachine.done(machineKeys.machineDoneKey);
	}

	public static void test_globalExit(PrintStream trace,
			CrashingReport crashingReport) {
		if (trace != null)
			trace.println("--- test_globalExit");
		MachineKeys machineKeys = new MachineKeys(); 
		ReactiveMachine reactiveMachine =
			ReactiveFactoryImpl.REACTIVE_FACTORY.newReactiveMachine(
					BREATHE_DURATION_MS, crashingReport, machineKeys);
		long globalExitKey = reactiveMachine.newKey();
		if (trace != null)
			trace.println("GlobalExitKey " + globalExitKey);
		ReactiveCommunity community;
		{
			/*
			 * Launches a CommunityManagerAgent.
			 */
			LaunchingContext launchingContext =
				new LaunchingContextImpl();
			reactiveMachine.launchAgent(
					machineKeys.agentLaunchingKey,
					new CommunityManagerAgent(null, null,
						globalExitKey, reactiveMachine.newKey()),
					launchingContext,
					ReactiveMachine.PROFILE_FULL);
			while (launchingContext.getArgumentValue(
					Agent.ARG_DESTINATION_COMMUNITY) == null)
				reactiveMachine.breathe("CommunityBeOnLine");
			community =
				(ReactiveCommunity)launchingContext.getArgumentValue(
					Agent.ARG_DESTINATION_COMMUNITY);
		}
		{
			/*
			 * Launches an GlobalExitAgent with a bad key.
			 */
			reactiveMachine.launchAgent(
					machineKeys.agentLaunchingKey,
					new GlobalExitAgent(globalExitKey + 1),
					new LaunchingContextImpl(community),
					ReactiveMachine.PROFILE_STANDARD);
		}
		{
			/*
			 * Launches an GlobalExitAgent with a good key.
			 */
			reactiveMachine.launchAgent(
					machineKeys.agentLaunchingKey,
					new GlobalExitAgent(globalExitKey),
					new LaunchingContextImpl(community),
					ReactiveMachine.PROFILE_STANDARD);
		}
		/*
		 * Waits for done.
		 */
		reactiveMachine.done(machineKeys.machineDoneKey);
	}

	public static void test_clock(PrintStream trace,
			CrashingReport crashingReport) {
		if (trace != null)
			trace.println("--- test_clock");
		MachineKeys machineKeys = new MachineKeys(); 
		ReactiveMachine reactiveMachine =
			ReactiveFactoryImpl.REACTIVE_FACTORY.newReactiveMachine(
					BREATHE_DURATION_MS, crashingReport, machineKeys);
		long globalExitKey = reactiveMachine.newKey();
		ReactiveCommunity community;
		{
			/*
			 * Launches a CommunityManagerAgent.
			 */
			LaunchingContext launchingContext =
				new LaunchingContextImpl();
			reactiveMachine.launchAgent(
					machineKeys.agentLaunchingKey,
					new CommunityManagerAgent(null, null,
							globalExitKey, reactiveMachine.newKey()),
					launchingContext,
					ReactiveMachine.PROFILE_FULL);
			while (launchingContext.getArgumentValue(
					Agent.ARG_DESTINATION_COMMUNITY) == null)
				reactiveMachine.breathe("CommunityBeOnLine");
			community =
				(ReactiveCommunity)launchingContext.getArgumentValue(
					Agent.ARG_DESTINATION_COMMUNITY);
		}
		{
			/*
			 * Launches a ClockAgent.
			 */
			reactiveMachine.launchAgent(
					machineKeys.agentLaunchingKey,
					new ClockAgent(globalExitKey, reactiveMachine.newKey(), 50),
					new LaunchingContextImpl(community),
					ReactiveMachine.PROFILE_STANDARD);
		}
		{
			/*
			 * Launches a GlobalExitAgent that waits 3 clock events before exit.
			 */
			GlobalExitAgent exitAgent = new GlobalExitAgent(globalExitKey) {
				private int clockCount = 0;
				
				public boolean continues() {
					return this.clockCount < 3;
				}
	
				public boolean processEvent(Event event) {
					if (event.getName() == "clock".intern()) {
						this.clockCount += 1;
						return true;
					}
					return false;
				}
			};
			reactiveMachine.launchAgent(
					machineKeys.agentLaunchingKey,
					exitAgent,
					new LaunchingContextImpl(community),
					ReactiveMachine.PROFILE_STANDARD);
		}
		/*
		 * Waits for done.
		 */
		reactiveMachine.done(machineKeys.machineDoneKey);
	}

	public static void test_conditionalEvent(PrintStream trace,
			CrashingReport crashingReport) {
		if (trace != null)
			trace.println("--- test_conditionalEvent");
		MachineKeys machineKeys = new MachineKeys(); 
		ReactiveMachine reactiveMachine =
			ReactiveFactoryImpl.REACTIVE_FACTORY.newReactiveMachine(
					BREATHE_DURATION_MS, crashingReport, machineKeys);
		long globalExitKey = reactiveMachine.newKey();
		ReactiveCommunity community;
		{
			/*
			 * Launches a FacetsAgent with an EventBoolFacet.
			 */
			FacetsAgent mainAgent =
				new FacetsAgent() {
					private ExitFacet exitFacet() {
						return (ExitFacet)this.facetList.get(0);
					}
					private ConditionalEventFacet eventBoolFacet() {
						return (ConditionalEventFacet)this.facetList.get(1);
					}
					protected ReactiveCommunity newCommunity() {
						return this.reac.newCommunity(null);
					}
					public void init(LaunchingContext launchingContext) {
						super.init(launchingContext);
						this.addFacet(new ExitFacet());
						this.addFacet(new ConditionalEventFacet());
						{
							long globalExitKey =
								((Long)launchingContext.getArgumentValue(
									"globalExitKey")).longValue();
							this.exitFacet().setExitKeys(
								globalExitKey, globalExitKey);
						}
						{
							Event e1 = this.reac.searchEvent("e1");
							Event e2 = this.reac.searchEvent("e2");
							Event e3 = this.reac.searchEvent("e3");
							BoolCondition expr =
								new OrBoolCondition(
									new PositiveEventBoolCondition(e1),
									new NegativeEventBoolCondition(e2));
							this.eventBoolFacet().setExpression(expr, e3, true);
						}
					}
					public boolean continues() {
						return !this.exitFacet().isExit();
					}
					public void terminate() {
						super.terminate();
						this.reac.closeCommunity();
						this.reac.disconnect();
					}
				};
			LaunchingContext launchingContext =
				new LaunchingContextImpl();
			launchingContext.addArgument(
				"globalExitKey", new Long(globalExitKey));
			reactiveMachine.launchAgent(
					machineKeys.agentLaunchingKey,
					mainAgent,
					launchingContext,
					ReactiveMachine.PROFILE_FULL);
			while (launchingContext.getArgumentValue(
					Agent.ARG_DESTINATION_COMMUNITY) == null)
				reactiveMachine.breathe("CommunityBeOnLine");
			community =
				(ReactiveCommunity)launchingContext.getArgumentValue(
					Agent.ARG_DESTINATION_COMMUNITY);
		}
		{
			/*
			 * Launches an GlobalExitAgent.
			 */
			GlobalExitAgent exitAgent = new GlobalExitAgent(globalExitKey) {
				public boolean continues() {
					Event noEvent = this.reac.createEvent();
					Event e2 = this.reac.searchEvent("e2");
					this.reac.stop();
					this.reac.generateEvent(e2);
					/*
					 * This event generation makes that e3 will be
					 *   not generated at the next instant.
					 */
					this.reac.stop();
					this.reac.generateEvent(noEvent);
					/*
					 * This event generation avoids a deadlock
					 *   caused by mainAgent.waitEvent().
					 */
					this.reac.stop();
					return false;
				}
			};
			reactiveMachine.launchAgent(
					machineKeys.agentLaunchingKey,
					exitAgent,
					new LaunchingContextImpl(community),
					ReactiveMachine.PROFILE_STANDARD);
		}
		/*
		 * Waits for done.
		 */
		reactiveMachine.done(machineKeys.machineDoneKey);
	}

	public static void test_Syracuse(PrintStream trace,
			CrashingReport crashingReport) {
		if (trace != null)
			trace.println("--- test_Syracuse");
		MachineKeys machineKeys = new MachineKeys(); 
		ReactiveMachine reactiveMachine =
			ReactiveFactoryImpl.REACTIVE_FACTORY.newReactiveMachine(
					BREATHE_DURATION_MS, crashingReport, machineKeys);
		/*
		 * Launches a main SyracuseAgent.
		 */
		reactiveMachine.launchAgent(
				machineKeys.agentLaunchingKey,
				new SyracuseAgent(true, 1, 7, new End()),
				null,
				ReactiveMachine.PROFILE_FULL);
		/*
		 * Waits for done.
		 */
		reactiveMachine.done(machineKeys.machineDoneKey);
	}

	public static void test_board(PrintStream trace,
			CrashingReport crashingReport) {
		if (trace != null)
			trace.println("--- test_board");
		MachineKeys machineKeys = new MachineKeys(); 
		ReactiveMachine reactiveMachine =
			ReactiveFactoryImpl.REACTIVE_FACTORY.newReactiveMachine(
					BREATHE_DURATION_MS, crashingReport, machineKeys);
		long globalExitKey = reactiveMachine.newKey();
		ReactiveCommunity community;
		{
			/*
			 * Launches a CommunityManagerAgent.
			 */
			CommunityManagerAgent mainAgent =
				new CommunityManagerAgent(
					"turtles", "boards/turtles_board.xml",
					globalExitKey, reactiveMachine.newKey());
			LaunchingContext launchingContext =
				new LaunchingContextImpl();
			reactiveMachine.launchAgent(
					machineKeys.agentLaunchingKey,
					mainAgent,
					launchingContext,
					ReactiveMachine.PROFILE_FULL);
			while (launchingContext.getArgumentValue(
					Agent.ARG_DESTINATION_COMMUNITY) == null)
				reactiveMachine.breathe("CommunityBeOnLine");
			community =
				(ReactiveCommunity)launchingContext.getArgumentValue(
					Agent.ARG_DESTINATION_COMMUNITY);
		}
		{
			/*
			 * Launches an GlobalExitAgent that creates the "turtles" project.
			 */
			GlobalExitAgent exitAgent = new GlobalExitAgent(globalExitKey) {
				public boolean continues() {
					ReactiveCommunity community = this.reac.getCommunity();
					ParentElement boardElement =
						(ParentElement)community.getBoard();
					ParentElement projectsElement =
						(ParentElement)boardElement.getChild(
								Element.REACTIVE_NAMESPACE, "projects");
					/*
					 * Refused proposition of creating a "wiki" child of "board".
					 */
					this.reac.generateActionMessage(
							new ElementAppendAction(
								(ElementWritingAccess)boardElement,
								boardElement,
								new ParentElementImpl(
										Element.REACTIVE_NAMESPACE, "wiki"),
								ElementActionImpl.STATUS_PROPOSED));
					/*
					 * Accepted proposition of creating a "turtles" project.
					 */
					this.reac.generateActionMessage(
							new ElementAppendAction(
								(ElementWritingAccess)boardElement,
								projectsElement,
								new ParentElementImpl(
										Element.REACTIVE_NAMESPACE, "turtles"),
								ElementActionImpl.STATUS_PROPOSED));
					this.reac.stop();
					this.reac.stop();
					this.reac.stop();
					return false;
				}
			};
			reactiveMachine.launchAgent(
					machineKeys.agentLaunchingKey,
					exitAgent,
					new LaunchingContextImpl(community),
					ReactiveMachine.PROFILE_STANDARD);
		}
		/*
		 * Waits for done.
		 */
		reactiveMachine.done(machineKeys.machineDoneKey);
	}
}
